#ifndef ADWIN_MACROS_H
#define ADWIN_MACROS_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/***********************************************************************
 * handling debug messages:                                            *
 * calls should be made like this:                                     *
 * PRINTDEBUG(DBG_CONFIGFILE, ("could not open file '%s'", filename)); *
 * if DBG_CONFIGFILE is defined, this will be printed,                 *
 * otherwise nothing will happen.                                      *
 ***********************************************************************/
#include <stdio.h>
/* Print debug messages from generic ethernet driver */
#define DBG_GENERIC_DRV   0
/* Print debug messages for config file handling */
#define DBG_CONFIGFILE    0
/* Print debug messages for semaphore access */
#define DBG_SEMAPHORES    0
/* Print debug messages for shared memory access */
#define DBG_SHARED_MEM    0
/* Print debug messages for libadwin */
#define DBG_LIBADWIN      0

#define PRINTDEBUG(do_debug, msg) \
    if (do_debug) printf msg

extern int initialized;
#define MAX_ERRSTR_LEN 256



/* since we have little-endian as byte order in the ADwin protocol, we
 * work similar to ntohl and htonl, just the other way. Not that
 * WORDS_BIGENDIAN is defined in config.h, so if you are using the
 * generic driver, you may have to adjust WORDS_BIGENDIAN.
 */


#if (defined(__APPLE__) && defined(__BIG_ENDIAN__)) || (!defined(__APPLE__) && defined(WORDS_BIGENDIAN))
#define ADWIN_BIG_ENDIAN
#else
#define ADWIN_LITTLE_ENDIAN
#endif

#ifndef __bswap_16
#define __bswap_16(x) \
    ((((x) >> 8) & 0xff) | (((x) & 0xff) << 8))
#endif
#ifndef __bswap_32
#define __bswap_32(x) \
    ((((x) & 0xff000000) >> 24) | (((x) & 0x00ff0000) >>  8) | \
     (((x) & 0x0000ff00) <<  8) | (((x) & 0x000000ff) << 24))
#endif
#ifdef ADWIN_BIG_ENDIAN
#define letohl(x)   __bswap_32(x)
#define letohs(x)   __bswap_16(x)
#define htolel(x)   __bswap_32(x)
#define htoles(x)   __bswap_16(x)

#else
#  define letohl(x) (x)
#  define letohs(x) (x)
#  define htolel(x) (x)
#  define htoles(x) (x)
#endif

#if ! (defined (WIN32) || defined (__BORLANDC__))

#  define SEM_KEY 0x41447769 // This is ADwi in Hex :-)
// the device specific semaphores are at SEM_KEY
// the global semaphore is at SEM_KEY + 1
// the retry semaphore is at SEM_KEY + 2

#  define SHM_KEY 0x41447796

/* This ugly stuff seems to be necessary for semaphores... */
#  include <time.h>
#  include <sys/sem.h>
#  if (defined(__GNU_LIBRARY__) || defined(__FreeBSD__) || defined(__APPLE__)) && !defined(_SEM_SEMUN_UNDEFINED)
/* union semun is defined by including <sys/sem.h> */
#  else
/* according to X/OPEN we have to define it ourselves */
union semun {
    int val;                  /* value for SETVAL */
    struct semid_ds *buf;     /* buffer for IPC_STAT, IPC_SET */
    unsigned short *array;    /* array for GETALL, SETALL */
    /* Linux specific part: */
    struct seminfo *__buf;    /* buffer for IPC_INFO */
};
#  endif

#endif /* Win32 || BORLANDC */

#endif /* ADWIN_MACROS_H */
